         ---------------------------
         -- Mighty Mouse Routines --   "Here I come to save the day!"
         ---------------------------
-- You need DOS mouse support for these routines.

-- modified June 20, 1998 by Pete Eberlein <xseal@harborside.com>
--   get_mouse() returns mouse events as usual
--   the first call to mouse_pointer activates the extended mouse
-- important: this version of mighty mouse redefines the pixel and get_pixel
--   routines, so that you do not need to call mouse_pointer when drawing 
--   graphics to the screen!  For all graphic routines to work this way,
--   (such as display_image in image.e) they must be included after mighty.e.
--   (routines that write directly to screen memory will still need to use
--     mouse_pointer to hide and show the mouse cursor.)
-- added:
--   mouse_inside(x1, y1, x2, y2)
--     returns true if the mouse is in the bounding box (x1,y1)-(x2,y2)
--   mouse_enter(x1, y1, x2, y2)
--     returns true if the mouse entered the bounding box (x1,y1)-(x2,y2)
--   mouse_leave(x1, y1, x2, y2)
--     returns true if the mouse left the bounding box (x1,y1)-(x2,y2)
--   poll_mouse()
--     returns {buttons, x_pos, y_pos} of the mouse
--     bit masks for buttons:  1-left  2-right  4-middle
--
--   set_mouse_cursor({map, mask, hotspot})
--     mask is AND'ed to screen, then map is XOR'ed on top
--      (map and mask must be exactly the same size)
--     hotspot is {dx,dy} displacement of mouse relative to image
--   set_mouse_cursor(default_cursor)
--     or you can use one of the following default cursors:

-- MODIFIED BY EUMAN
-- DEC 12, 2000
-- mdmighty.e for use with ZLoop.e by Euman

include machine.e

global constant
    IDC_APPSTARTING = 13, -- Standard arrow and small hourglass
    IDC_ARROW = 1,    --  Standard arrow
    IDC_CROSS = 3,  -- Crosshair
    IDC_IBEAM = 4, -- Text I-beam
    IDC_ICON = 5,  -- Empty icon
    IDC_NO = 6, -- Slashed circle
    IDC_SIZE = 7,  -- Four-pointed arrow
    IDC_SIZEALL = 7, -- Same as MC_SIZE
    IDC_SIZENESW = 8, -- Double-pointed arrow pointing northeast and southwest
    IDC_SIZENS = 9, -- Double-pointed arrow pointing north and south
    IDC_SIZENWSE  = 10, -- Double-pointed arrow pointing northwest and southeast
    IDC_SIZEWE = 11, -- Double-pointed arrow pointing west and east
    IDC_UPARROW = 12, -- Vertical arrow
    IDC_WAIT = 2 -- Hourglass

-- Mouse Event Numbers:
global constant MOVE = 1,       -- track every movement of the mouse
                LEFT_DOWN = 2,  -- the rest are button pressing/releasing
                LEFT_UP = 4,
                RIGHT_DOWN = 8,
                RIGHT_UP = 16,
                MIDDLE_DOWN = 32,
                MIDDLE_UP = 64

constant M_GET_MOUSE = 14,
         M_MOUSE_EVENTS = 15,
         M_MOUSE_POINTER = 24

integer extended_mouse
    extended_mouse = 0

sequence m_back, m_pos, m_bord, m_map, m_mask, m_hotspot, m_lastpos, limin, limax
integer m_visible, m_buts, m_events, m_moved
integer mWide, mTall
global sequence default_cursors

type point(sequence s)
    return length(s)=2 and atom(s[1]) and atom(s[2])
end type

--global procedure limit_mouse(point min,point max)
    -- Limit Mouse to rectangle.
--    limin=min limax=max
--end procedure

--limit_mouse({1,100},{1016,752})

global function poll_mouse()
    sequence REGS, temp, pos
    integer dy, dx, bx1, bx2, bx3, bx4, tx1, tx2

    REGS = repeat(0, 10)
    REGS[REG_AX] = #B              -- get relative motion counters
    REGS = dos_interrupt(#33, REGS)
    temp = REGS[REG_CX] & REGS[REG_DX]
    m_lastpos = m_pos                
    m_pos += temp - 65536 * (temp > 32767)
    m_pos = m_pos * (m_pos >= 0) * (m_pos <= m_bord) + m_bord * (m_pos > m_bord)
    --m_pos=m_pos*(m_pos>=limin)+limin*(m_pos<limin)
    --m_pos=m_pos*(m_pos<=limax)+limax*(m_pos>limax)

    REGS[REG_AX] = 3               -- get mouse buttons in BX
    REGS = dos_interrupt(#33, REGS)
    m_moved = (m_lastpos[1] != m_pos[1]) or (m_lastpos[2] != m_pos[2])
    if m_moved then
        if m_visible = 0 then
            temp = repeat(0, mTall)
            pos = (m_pos - m_hotspot) & mWide
            for y = 1 to mTall do
                temp[y] = get_pixel(pos)
                pos[2] += 1
            end for
            dx = m_pos[1] - m_lastpos[1]
            dy = m_pos[2] - m_lastpos[2]
            if (dy > -mTall and dy < mTall) 
            and (dx > -mWide and dx < mWide) then
                if dx >= 0 and dx < mWide then   -- mouse moved to the right
                    tx1 = 1           tx2 = mWide - dx
                    bx1 = 1 + dx      bx2 = mWide
                    bx3 = 1           bx4 = dx
                    dx = 0
                elsif dx > -mWide and dx < 0 then -- mouse moved to the left
                    tx1 = 1 - dx      tx2 = mWide
                    bx1 = 1           bx2 = mWide + dx
                    bx3 = mWide+1+dx  bx4 = mWide
                    dx += mWide 
                end if
                dy = - dy
                pos = m_lastpos - m_hotspot
                for y = 1 to length(m_back) do
                    dy += 1
                    if dy >= 1 and dy <= mTall then
                        temp[dy][tx1..tx2] = m_back[y][bx1..bx2]
                        pixel(m_back[y][bx3..bx4], pos+{dx,y-1})
                    else
                        pixel(m_back[y], pos+{0,y-1})
                    end if
                end for
            else  -- new mouse image doesn't overlap old background
                pos = m_lastpos-m_hotspot
                for y = 1 to length(m_back) do
                    pixel(m_back[y], pos)
                    pos[2] += 1 
                end for
            end if
            pos = m_pos-m_hotspot
            for y = 1 to mTall do
                pixel(xor_bits(and_bits(temp[y], m_mask[y]), m_map[y]), pos)
                pos[2] += 1 
            end for
            m_back = temp
        end if
    end if
    return REGS[REG_BX] & m_pos
end function


global procedure mouse_events(integer events)
-- select the mouse events to be reported by get_mouse()
-- e.g. mouse_events(LEFT_UP + LEFT_DOWN + RIGHT_DOWN)
    m_events = events
    if not extended_mouse then
        machine_proc(M_MOUSE_EVENTS, events)
    end if
end procedure

global procedure mouse_pointer(integer show_it)
-- show (1) or hide (0) the mouse pointer
    sequence vparams, regs, pos
    if not extended_mouse then
        vparams = video_config()
    --    if (vparams[VC_XPIXELS] or vparams[VC_YPIXELS]) = 0 then
            -- we're in text mode, so
    --        machine_proc(M_MOUSE_POINTER, show_it)
    --    end if
        
        regs = repeat(0, 10)
        regs = dos_interrupt(#33, regs) -- reset mouse
    --   if regs[REG_AX] != #FFFF then
    --        return   -- no mouse
    --    end if

        extended_mouse = 1
        m_bord = {vparams[VC_XPIXELS], vparams[VC_YPIXELS]} - 1
    
        m_pos = floor((m_bord+1)/2)

        m_lastpos = m_pos
        m_back = {{}}
        m_events = MOVE + LEFT_DOWN + LEFT_UP 
                        + RIGHT_DOWN + RIGHT_UP 
                        + MIDDLE_DOWN + MIDDLE_UP
        m_visible = 1  -- not yet visible
        m_buts = 0     -- no buttons pressed

        m_map = default_cursors[1][1]
        m_mask = default_cursors[1][2]
        m_hotspot = default_cursors[1][3]

        mWide = length(m_map[1])
        mTall = length(m_map)
    end if
    if show_it then

        if m_visible > 0 then
            m_visible -= 1 --m_visible - 1
        end if
        if m_visible = 0 then
            
            m_back = repeat(0, mTall)
            pos = (m_pos - m_hotspot) & mWide
            for y = 1 to mTall do
                m_back[y] = get_pixel(pos)
                pos[2] += 1 
            end for
            pos = m_pos - m_hotspot
            for y = 1 to mTall do
                pixel(xor_bits(and_bits(m_back[y], m_mask[y]), m_map[y]), pos)
                pos[2] += 1 
            end for
        end if
    else
        if m_visible = 0 then
            pos = m_pos - m_hotspot
            

            for y = 1 to length(m_back) do
                pixel(m_back[y], pos) 
                pos[2] += 1 
            end for
        end if
        m_visible += 1 
    end if
end procedure

global procedure set_mouse_cursor(object cursor)
    mouse_pointer(0)
    if sequence(cursor) then
        m_map = cursor[1]
        m_mask = #FF * (cursor[2] != 0)
        m_hotspot = {0,0}
        if length(cursor) > 2 then
            if sequence(cursor[3]) then
                if length(cursor[3]) = 2 then
                    m_hotspot = cursor[3]
                end if
            end if
        end if
    else
        m_map = default_cursors[cursor][1]
        m_mask = default_cursors[cursor][2]
        m_hotspot = default_cursors[cursor][3]
    end if        

    mWide = length(m_map[1])
    mTall = length(m_map)
    mouse_pointer(1)
end procedure

global function mouse_inside(integer x1, integer y1, integer x2, integer y2)
    return m_pos[1] >= x1
       and m_pos[2] >= y1
       and m_pos[1] <= x2
       and m_pos[2] <= y2
end function

global function mouse_enter(integer x1, integer y1, integer x2, integer y2)
    return m_pos[1] >= x1
       and m_pos[2] >= y1
       and m_pos[1] <= x2
       and m_pos[2] <= y2
       and (m_lastpos[1] < x1 
       or m_lastpos[2] < y1 
       or m_lastpos[1] > x2 
       or m_lastpos[2] > y2)
end function

global function mouse_leave(integer x1, integer y1, integer x2, integer y2)
    return m_lastpos[1] >= x1
       and m_lastpos[2] >= y1
       and m_lastpos[1] <= x2
       and m_lastpos[2] <= y2
       and (m_pos[1] < x1 
       or m_pos[2] < y1 
       or m_pos[1] > x2 
       or m_pos[2] > y2)
end function

default_cursors = {
{   ---- normal arrow ----
    {
"           ",
" >         ",
" >>        ",
" >>>       ",
" >>>>      ",
" >>>>>     ",
" >>>>>>    ",
" >>>>>>>   ",
" >>>>>>>>  ",
" >>>>>     ",
" >> >>     ",
" >   >>    ",
"     >>    ",
"      >>   ",
"      >>   ",
"           "}-32,
    #FF*(' '={
"xx         ",
"x>x        ",
"x>>x       ",
"x>>>x      ",
"x>>>>x     ",
"x>>>>>x    ",
"x>>>>>>x   ",
"x>>>>>>>x  ",
"x>>>>>>>>x ",
"x>>>>>xxxxx",
"x>>x>>x    ",
"x>x x>>x   ",
"xx  x>>x   ",
"x    x>>x  ",
"     x>>x  ",
"      xxx  "}),
    {0,0}
}}